/** Class for representing the base combinations.
 *  This is used by the DNA class to store the sequence information
 *  it is also used to convert user input Strings to base
 *  pairs.  There is no public constructor in this class because
 *  there are exactly four orientations of base pairs.
 *  @author mwa
 */
public class BasePair{

    // public constants representing the 4 base pair orientations
    public static final BasePair AT = new BasePair('A','T');
    public static final BasePair TA = new BasePair('T','A');
    public static final BasePair CG = new BasePair('C','G');
    public static final BasePair GC = new BasePair('G','C');

    private char one; // first component in pair
    private char two; // second component in pair

    /** Private constructor which takes the two components
     *  of a base pair.
     *  @param first component
     *  @param second component
     */
    private BasePair(char one, char two){
	this.one = one;
	this.two = two;
    }

    /** @return the first base component
     */
    public char getOne(){
	return one;
    }
    
    /** @return the second base component
     */
    public char getTwo(){
	return two;
    }

    /** Converts a character representation of a base
     *  into the object representation, if the character
     *  does not represent a base, null is returned
     *  @param the base
     *  @return the BasePair representing the base
     */
    public static BasePair getBasePair(char one){
	switch(one){
	case 'A':
	    return AT;
	case 'G':
	    return GC;
	case 'C':
	    return CG;
	case 'T':
	    return TA;
	default:
	    return null;
	}
    }

    /** @return the BasePair representing the complement of 
     *  the current BasePair
     */
    public BasePair complement(){
	if (this == AT)
	    return TA;
	else if (this == TA)
	    return AT;
	else if (this == CG)
	    return GC;
	return CG;
    }

    /** @param object to check against
     *  @return true iff they are referencing the same thing
     */
    public boolean equals(Object o){
	return o == this;
    }

    /** @return the String version of the first base 
     */
    public String toString(){
	return "" + one;
    }


}
