/** This is the class that represent the node in our doubly linked list
 *  Has previous and next pointer and contains a generic element.
 *  This class is only used by the DLList class.
 *  @author mwa.
 */

public class DLLNode<E>{

    private DLLNode prev;  // pointer to previous node
    private DLLNode next;  // pointer to next node
    private E el;          // data element stored here

    /** Construct the node with no neighbors
     *  @param element to store 
     */
    public DLLNode(E el){
	this.el = null;
	prev = null;
	next = null;
    }

    /** Construct the node with 2 neighbors
     *  @param element to store
     *  @param pointer to previous neighbor
     *  @param pointer to next neighbor
     */
    public DLLNode(E el, DLLNode prev, DLLNode next){
	this.el = el;
	this.prev = prev;
	this.next = next;
    }

    /** Sets the next pointer to the parameter
     *  @param pointer to next neighbor
     */
    public void setNext(DLLNode next){
	this.next = next;
    }

    /** Sets the previous pointer to the parameter
     *  @param pointer to previous neighbor
     */
    public void setPrev(DLLNode prev){
	this.prev = prev;
    }

    /** Sets the element
     *  @param the new element to store
     */
    public void setElement(E el){
	this.el = el;
    }

    /**  @return pointer to next neighbor
     */
    public DLLNode getNext(){
	return next;
    }

    /** @return pointer to previous neighbor
     */
    public DLLNode getPrev(){
	return prev;
    }
    
    /** @return element stored here
     */
    public E getElement(){
	return el;
    }
}

