/** Main class for doubly linked list.  Uses DLLNode to 
 *  form the list.  Keeps track of the current node in 
 *  focus, the size of the list and the direction we are
 *  currently accessing the list in.  Uses a generic 
 *  element type with DLLNode.
 *  @author mwa
 */
public class DLList<E>{

    private DLLNode<E> curr;  // current node in focus
    private int size;         // size of the list
    private int direction;    // access direction

    /** Constructs an empty list
     */
    public DLList(){
	curr = null;
	size = 0;
	direction = 1;
    }

    /** @return size of the list
     */
    public int getSize(){
	return size;
    }

    /** @return true iff the list is empty
     */
    public boolean isEmpty(){
	return curr == null;
    }

    /** @return the element stored at the focus, if no
     *	element exists it returns null
     */
    public E getCurrElement(){
	if (curr != null)
	    return curr.getElement();
	return null;
    }
    /** Sets the focused element to a new value
     *  @param new element value
     */
    public void setCurrElement(E el){
	if (curr != null)
	    curr.setElement(el);
    }

    /** Reverses the direction of access, subsequence
     *  accesses are made in the opposite direction
     */
    public void reverse(){
	direction = -direction;
    }

    /** Shifts the focus by dir in the current direction,
     *  negative value will shift the other way.
     *  @param amount to shift
     */
    public void translate(int dir){

	for (int i = 0; i < Math.abs(dir); i++)
	    if (direction*dir < 0)
	    curr = curr.getPrev();
	    else
		curr = curr.getNext();
    }
    
    /** Removes the focused element and makes the new
     *  focus the node immediately after it.
     *  @return true iff an element was removed
     */
    public boolean remove(){	
	if (curr == null)  // no elements left
	    return false;
	if (size == 1){  // last element left
	    curr = null;
	} else {
	    curr.getNext().setPrev(curr.getPrev());
	    curr.getPrev().setNext(curr.getNext());
	    if (direction < 0)
		curr = curr.getPrev();
	    else
		curr = curr.getNext();
	}
	size--;
	return true;
    }

    /** Inserts an element into the list immediately before
     *  the focus ("before" depends on current direction),
     *  the focus is set to the inserted node.
     *  @param the element to insert
     */
    public void insert(E el){
	DLLNode<E> n;
	if (curr == null){ // the list is empty
	    n = new DLLNode(el,null,null);
	    n.setNext(n);
	    n.setPrev(n);
	} else {
	    if (direction > 0){ // insert before
		n = new DLLNode(el,curr.getPrev(),curr);
		curr.getPrev().setNext(n);
		curr.setPrev(n); 
	    } else {  // insert after
		n = new DLLNode(el,curr,curr.getNext());
		curr.getNext().setPrev(n);
		curr.setNext(n);
	    }
	}
	curr = n;  // set focus
	size++;
    }

}
